"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.getStore = getStore;
exports.getStoreDispatch = getStoreDispatch;

var _toConsumableArray2 = _interopRequireDefault(require("@babel/runtime/helpers/toConsumableArray"));

var _defineProperty2 = _interopRequireDefault(require("@babel/runtime/helpers/defineProperty"));

var _slicedToArray2 = _interopRequireDefault(require("@babel/runtime/helpers/slicedToArray"));

var _redux = require("redux");

var _wangctUtil = require("wangct-util");

var _history = _interopRequireDefault(require("./history"));

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(source, true).forEach(function (key) { (0, _defineProperty2["default"])(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(source).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

/**
 * 获取store对象
 * @returns {any}
 */
function getStore(models) {
  var _this = this;

  models = (0, _wangctUtil.toAry)(models);
  var watchPropsMap = getWatchPropsMap(models);
  var store = (0, _redux.createStore)(function (state, action) {
    var _split = (action.type || '').split('/'),
        _split2 = (0, _slicedToArray2["default"])(_split, 2),
        namespace = _split2[0],
        funcField = _split2[1];

    var _ref = models.find(function (item) {
      return item.namespace === namespace;
    }) || {},
        _ref$reducers = _ref.reducers,
        reducers = _ref$reducers === void 0 ? {} : _ref$reducers,
        _ref$effects = _ref.effects,
        effects = _ref$effects === void 0 ? {} : _ref$effects;

    var updateState = {};

    if (!reducers.update) {
      reducers.update = update;
    }

    if (!reducers.updateField) {
      reducers.updateField = update;
    }

    if (effects[funcField]) {
      var gener = effects[funcField](action, {
        put: put.bind(_this, namespace),
        select: select,
        call: call
      });
      setTimeout(function () {
        loopGenerator(gener);
      }, 0);
    }

    if (reducers[funcField]) {
      var scopeState = reducers[funcField](state[namespace], action);
      var oldScopeState = state[namespace];

      if (scopeState._type === 'init') {
        scopeState = scopeState.data;
      }

      var watchState = getWatchState(scopeState, watchPropsMap[namespace], oldScopeState);
      updateState[namespace] = _objectSpread({}, oldScopeState, {}, scopeState, {}, watchState);
    }

    return _objectSpread({}, state, {}, updateState);
  }, {});
  /**
   * dispatch
   * @param namespace
   * @param action
   * @returns {Promise<any>}
   */

  function put(namespace, action) {
    getStoreDispatch(store, namespace)(action);
    return Promise.resolve(action);
  }
  /**
   * 获取state
   * @param func
   * @returns {Promise<any>}
   */


  function select(func) {
    return Promise.resolve(func(store.getState()));
  }
  /**
   * 执行异步函数
   * @param args
   * @returns {Promise<*[]>|*}
   */


  function call() {
    for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    var target = args[0];

    if ((0, _wangctUtil.isPromise)(target)) {
      return target;
    } else if ((0, _wangctUtil.isFunc)(target)) {
      return target.apply(void 0, (0, _toConsumableArray2["default"])(args.slice(1)));
    } else {
      return Promise.resolve(args);
    }
  }

  models.forEach(function (model) {
    var subscriptions = model.subscriptions,
        namespace = model.namespace;
    store.dispatch({
      type: model.namespace + '/update',
      field: 'multiple',
      data: model.state
    });

    if (subscriptions) {
      Object.keys(subscriptions).forEach(function (key) {
        (0, _wangctUtil.callFunc)(subscriptions[key], {
          dispatch: getStoreDispatch(store, namespace),
          history: _history["default"]
        });
      });
    }
  });
  return store;
}
/**
 * 获取store的dispatch
 * @param store
 * @param namespace
 * @returns {function(...[*]=)}
 */


function getStoreDispatch(store) {
  var namespace = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 'global';
  return function (action) {
    store.dispatch(_objectSpread({}, action, {
      type: formatType(action.type, namespace)
    }));
  };
}
/**
 * 格式化类型
 * @param type
 * @param namespace
 * @returns {string}
 */


function formatType() {
  var type = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : '';
  var namespace = arguments.length > 1 ? arguments[1] : undefined;

  var _type$split = type.split('/'),
      _type$split2 = (0, _slicedToArray2["default"])(_type$split, 2),
      typespace = _type$split2[0],
      funcField = _type$split2[1];

  return funcField ? type : namespace + '/' + typespace;
}
/**
 * 循环遍历生成器
 * @param gener
 * @param params
 */


function loopGenerator(gener, params) {
  var _gener$next = gener.next(params),
      value = _gener$next.value,
      done = _gener$next.done;

  if (!done) {
    if ((0, _wangctUtil.isPromise)(value)) {
      value.then(function (data) {
        loopGenerator(gener, data);
      });
    } else {
      loopGenerator(gener, value);
    }
  }
}
/**
 * 更新字段
 * @param state
 * @param field
 * @param data
 * @param parentField
 */


function update(state, _ref2) {
  var field = _ref2.field,
      data = _ref2.data,
      parentField = _ref2.parentField;
  var extState = field === 'multiple' ? data : (0, _defineProperty2["default"])({}, field, data);

  if (parentField) {
    extState = (0, _defineProperty2["default"])({}, parentField, _objectSpread({}, state[parentField], {}, extState));
  }

  return extState;
}
/**
 * 获取监听属性map
 * @param models
 */


function getWatchPropsMap(models) {
  return (0, _wangctUtil.aryToObject)(models, 'namespace', function (model) {
    var _model$watch = model.watch,
        watch = _model$watch === void 0 ? {} : _model$watch;
    var mapData = {};
    var objPropMapData = {};
    (0, _wangctUtil.objForEach)(watch, function (value, key) {
      var temp = key.split(',');
      temp.forEach(function (itemKey) {
        var data = itemKey.includes('.') ? objPropMapData : mapData;
        var ary = data[itemKey] || [];
        ary.push({
          func: value,
          args: temp
        });
        data[itemKey] = ary;
      });
    });
    return {
      propMap: mapData,
      objPropMap: objPropMapData
    };
  });
}
/**
 * 获取监听的state
 * @param state
 * @param watchPropsMap
 * @param originState
 */


function getWatchState(state, _ref4, originState) {
  var _ref4$propMap = _ref4.propMap,
      propMap = _ref4$propMap === void 0 ? {} : _ref4$propMap,
      _ref4$objPropMap = _ref4.objPropMap,
      objPropMap = _ref4$objPropMap === void 0 ? {} : _ref4$objPropMap;
  var watchState = {};
  var changedCache = {};
  var tempMap = new Map();

  var realState = _objectSpread({}, originState, {}, state);

  (0, _wangctUtil.objForEach)(state, function (value, key) {
    (0, _wangctUtil.toAry)(propMap[key]).forEach(function (item) {
      callWatchFunc(item);
    });
  });
  (0, _wangctUtil.objForEach)(objPropMap, function (value, key) {
    if (isChanged(key)) {
      value.forEach(function (item) {
        callWatchFunc(item);
      });
    }
  });
  /**
   * 字段值是否改变
   * @param field
   * @returns {boolean|*}
   */

  function isChanged(field) {
    if (changedCache[field] !== undefined) {
      return changedCache[field];
    }

    var result = getValueByWatchField(realState, field) !== getValueByWatchField(originState, field);
    changedCache[field] = result;
    return result;
  }
  /**
   * 调用监听函数
   * @param data
   */


  function callWatchFunc(data) {
    if (tempMap.get(data.func)) {
      return;
    }

    tempMap.set(data.func, true);
    var args = (0, _wangctUtil.toAry)(data.args).map(function (name) {
      var _name$split = name.split('.'),
          _name$split2 = (0, _slicedToArray2["default"])(_name$split, 1),
          field = _name$split2[0];

      return realState[field];
    });
    var funcState = data.func.apply(data, (0, _toConsumableArray2["default"])(args));
    watchState = _objectSpread({}, watchState, {}, funcState);
  }

  return watchState;
}
/**
 * 根据监听字段获取对应值
 * @param state
 * @param field
 * @returns {*}
 */


function getValueByWatchField() {
  var state = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
  var field = arguments.length > 1 ? arguments[1] : undefined;
  (0, _wangctUtil.toStr)(field).split('.').forEach(function (field) {
    if (state) {
      state = state[field];
    }
  });
  return state;
}