"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.request = request;
exports.setResValidator = setResValidator;
exports.setResFormatter = setResFormatter;

var _defineProperty2 = _interopRequireDefault(require("@babel/runtime/helpers/defineProperty"));

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(source, true).forEach(function (key) { (0, _defineProperty2["default"])(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(source).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

var _window = window,
    fetch = _window.fetch;
/**
 * 请求方法
 * @param url
 * @param options
 * @returns {*}
 */

function request(url) {
  var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
  options = formatOptions(options);
  var _options = options,
      _options$json = _options.json,
      json = _options$json === void 0 ? true : _options$json;
  return fetch(url, options).then(checkStatus).then(function (res) {
    return json ? res.json() : res;
  }).then(function (data) {
    var _options2 = options,
        _options2$validator = _options2.validator,
        validator = _options2$validator === void 0 ? resValidator : _options2$validator;

    if (json && validator) {
      return validator(data) ? resFormatter(data) : Promise.reject(data);
    }

    return data;
  });
}

var resValidator = function resValidator(data) {
  return data.code === 0;
};

var resFormatter = function resFormatter(data) {
  return data.data;
};
/**
 * 设置返回成功响应
 * @param func
 */


function setResValidator(func) {
  resValidator = func;
}
/**
 * 设置返回成功数据格式化
 * @param func
 */


function setResFormatter(func) {
  resFormatter = func;
}
/**
 * 格式化选项
 * @param options
 * @returns {*}
 */


function formatOptions(options) {
  var body = options.body;

  if (body && !(body instanceof FormData)) {
    options.body = JSON.stringify(options.body);
    options.headers = _objectSpread({}, options.headers, {
      'content-type': 'application/json'
    });
  }

  return options;
}
/**
 * 检测状态
 * @param response
 * @returns {*}
 */


function checkStatus(response) {
  var status = response.status;

  if (status >= 200 && status < 300) {
    return response;
  }

  var error = new Error(response.statusText);
  error.response = response;
  throw error;
}