"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.dispatch = dispatch;
exports.getDispatch = getDispatch;
exports.getGlobalConfig = getGlobalConfig;
exports.setGlobalConfig = setGlobalConfig;
exports.setRoutes = setRoutes;
exports.getRoutes = getRoutes;
exports.showLoading = showLoading;
exports.getElem = getElem;

var _toConsumableArray2 = _interopRequireDefault(require("@babel/runtime/helpers/toConsumableArray"));

var _wangctUtil = require("wangct-util");

var _react = _interopRequireDefault(require("react"));

var _Loading = _interopRequireDefault(require("wangct-react/lib/Loading"));

var _state = require("./state");

var _store = require("../modules/store");

var _dic = require("../json/dic");

/**
 * dispatch
 * @param args
 */
function dispatch() {
  return getDispatch().apply(void 0, arguments);
}
/**
 * 获取dispatch方法
 * @param namespace
 * @returns {function(...[*]=)}
 */


function getDispatch(namespace) {
  return (0, _store.getStoreDispatch)((0, _state.getStore)(), namespace);
}

var globalConfig = {};
/**
 * 获取全局配置
 */

function getGlobalConfig(key) {
  return key ? globalConfig[key] : globalConfig;
}
/**
 * 设置全局配置
 * @param key
 * @param value
 */


function setGlobalConfig(key, value) {
  if ((0, _wangctUtil.isDef)(value)) {
    globalConfig[key] = value;
  } else {
    globalConfig = key;
  }
}
/**
 * 更新路由
 * @param routes
 */


function setRoutes(routes) {
  dispatch({
    type: _dic.Fields.globalNamespace + '/updateField',
    field: 'routes',
    data: routes
  });
}
/**
 * 获取路由
 * @returns {*|{}|[{path: string, component: string}]|[{path: string, component: string}]|Route[]|Requireable<any[]>}
 */


function getRoutes() {
  return (0, _state.getFrameState)().routes || [];
}
/**
 * 显示加载中
 * @param promise
 * @param message
 * @returns {Q.Promise<T> | Promise<any> | Promise<T>}
 */


function showLoading(promise) {
  var message = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : '操作处理中，请稍候...';

  var _getFrameState = (0, _state.getFrameState)(),
      _getFrameState$fragme = _getFrameState.fragmentList,
      fragmentList = _getFrameState$fragme === void 0 ? [] : _getFrameState$fragme;

  var content = _react["default"].createElement(_Loading["default"], {
    loading: true,
    title: message
  });

  dispatch({
    type: 'updateField',
    field: 'fragmentList',
    data: [].concat((0, _toConsumableArray2["default"])(fragmentList), [content])
  });
  return (0, _wangctUtil.toPromise)(promise)["finally"](function () {
    var _getFrameState2 = (0, _state.getFrameState)(),
        _getFrameState2$fragm = _getFrameState2.fragmentList,
        fragmentList = _getFrameState2$fragm === void 0 ? [] : _getFrameState2$fragm;

    (0, _wangctUtil.aryRemove)(fragmentList, content);
    dispatch({
      type: 'updateField',
      field: 'fragmentList',
      data: (0, _toConsumableArray2["default"])(fragmentList)
    });
  });
}
/**
 * 获取元素
  * @param elem
 * @returns {HTMLElement}
 */


function getElem(elem) {
  return (0, _wangctUtil.isStr)(elem) ? document.getElementById(elem) : elem;
}